/*
 * getrange.c
 *
 * $Source: /nfs/source/MASTER/garden/datlib/RCS/getrange.c,v $
 * $Date: 94/05/17 01:44:22 $
 * $Author: klee $
 * $Locker: klee $
 *
 * This file is part of the dat library.
 *
 * $Log:	getrange.c,v $
 * Revision 1.6  94/05/17  01:44:22  klee
 * included string.h, changed index() to strchr()
 * 
 * Revision 1.5  90/12/08  03:58:23  kaboom
 * Replaced under RCS, moved copyright message to copyright.h.
 * 
 * Revision 1.4  89/01/13  02:35:46  jwang
 * Add M.I.T. copyright notice.
 * 
 * Revision 1.3  88/03/26  19:42:11  steve
 * Out-of-range args triggers error message and exits, preventing d_submatrix
 * from dumping core.
 * 
 * Revision 1.2  87/03/22  18:35:04  steve
 * Don't fix out-of-range values
 * 
 */

#ifndef lint
static char *rcsid = "$Header: /nfs/source/MASTER/garden/datlib/RCS/getrange.c,v 1.6 94/05/17 01:44:22 klee Exp Locker: klee $";
static char *rcssource = "$Source: /nfs/source/MASTER/garden/datlib/RCS/getrange.c,v $";
#endif

#include "copyright.h"
#include <ctype.h>
#include <stdio.h>
#include <string.h>

/* When CLIP is defined, values which fall out of range are fixed up. */
/* #define CLIP 1 */

/* Values returned by getrange() */
#define STARTSTOP	0
#define STARTSIZE	1
#define DELETEDIM	2

/* int getrange(program, arg, start, stop, max, step)
**
** Parse a string for range arguments.  Arguments must be of the form:
**	[start[:stop | +size]][/step]
** On entry:
**	program = program name
**	arg	= string to parse
**	*start	= default starting value
**	*stop	= default stopping value
**	*step	= default step value
**	max	= maximum value for *start and *stop
** On exit:
**	*start 	= parsed value, if any
**	*stop 	= parsed value, if any
**	*step 	= parsed value, if any
**	getrange= STARTSTOP if argument is in the form a:b
**		= STARTSIZE if argument is in the form a+b
**		= DELETEDIM if argument is in the form a
**
** Note:
**	The "stop" value returned is one larger than that parsed.  If the
** user specifies a:b, getrange returns with *start = a and *stop = b+1
** so that getrange() may be used with d_submatrix() without adjusting 
** any values.
*/
int getrange(program, arg, start, stop, max, step)
char *program, *arg;
long *start, *stop, max, *step;
{
    char *ptr;
    int status;
    long atol();

    if (isdigit(*arg)) {
	*start = atol(arg);
	*stop = *start + 1;
	*step = 1;
	status = DELETEDIM;
    }

    if ((ptr = strchr(arg, ':')) != NULL) {
	if (!isdigit(*++ptr)) {
	    *stop = max;
	}
	else {
	    *stop = atol(ptr) + 1;
	}
	status = STARTSTOP;
    }
    else if ((ptr = strchr(arg, '+')) != NULL) {
	if (!isdigit(*++ptr)) {
	    fprintf(stderr, "%s: Incorrectly specified size: %s\n", program, 
		    arg);
	    exit(-1);
	}
	*stop = atol(ptr) + *start;
	status = STARTSIZE;
    }

    if ((ptr = strchr(arg, '/')) != NULL) {
	if ((*step=atol(ptr+1)) <= 0) {
	    fprintf(stderr, "%s: Incorrectly specified step: %s\n", 
		    program, ptr);
	    exit(-1);
	}
    }

#ifdef CLIP
    if (*start < 0) *start = 0;
    if (*start > max-1) *start = max-1;
    if (*stop <= *start) *stop = *start+1;
    if (*stop > max) *stop = max;
    if (*step <= 0) *step = 1;
#else
	if (! (0<=*start && *start<*stop && *stop<=max)) {
	    fprintf(stderr, "%s: [%s] invalid (dimension is [0:%ld])\n", 
		    program, arg, max-1);
	    exit(-1);
	}
#endif
    return(status);
}

