/***************************************************************\
*   snd.h							*
*   A new soundfile interface layer				*
*   Header file							*
*   1993mar03  dpwe  after convo.c				*
\***************************************************************/

#ifndef _SND_H

#define _SND_H
/* std libs */
#include <dpwelib.h>
/* personal libs */
#include <sndf.h>

/* constants */
#define SND_BUFSZ	16384	/* frames held from disk */
#define SND_STDIO_PATH	"-"	/* this filename causes access on stdin/out */
#define SND_AUDIO_PATH	"@"	/* this filename connects to real-time IO */

/* typedefs */
typedef struct {	/* sound file and descriptor */
    char	*path;		/* file info */
    FILE	*file;
    char	mode;
    int		isStdio;	/* TRUE if file is pipe to stdin/stdout */
    int		isRealtime;	/* TRUE if connected to audIO, not file */
    SFSTRUCT	*snd;		/* raw header info read from file */
    long	frames;		/* fields derived from header */
    float	duration;
    int 	userFormat;	/* user sees data in this format */
    int 	userChans;	/* user sees this many channels of data */
    int 	flags;
    /* buffer unconverted frames from disk */
    short	*buf;
    long	bufSize;	/* actual frames space in *buf */
    long	bufCont;	/* amt of valid data in *buf */
    /* nice little pointer for dspB */
    void 	*data;		/* pointer to memory buffer with data !? */
} SOUND;

/* defaults for open-write (new) file */
#define DFLT_CHNS	2
#define DFLT_FMT	SFMT_SHORT
#define DFLT_SR		(44100.0)

SOUND *sndNew PARG((SOUND *snd));
    /* just allocate a new SOUND structure
       Clone format parameters (chans, srate) from <snd> if any. */
void sndFree PARG((SOUND *snd));
    /* destroy sound, including closing any file */
SOUND *sndOpen PARG((SOUND *snd, char *path, char *mode));
    /* open sound.  If <snd> is NULL, allocate new one.
       If <path> is NULL, take from snd as passed in (or err).
       <mode> is "r" or "w" only */
void sndClose PARG((SOUND *snd));
    /* close file attached to snd */
void sndPrint(SOUND *snd, FILE *stream, char *tag);
    /* print a text description of a snd struct */
long sndReadFrames PARG((SOUND *snd, void *buf, long frames));
    /* read <frames> sample frames as floats into the buffer,
       return number actually read. */
long sndWriteFrames PARG((SOUND *snd, void *buf, long frames));
    /* write <frames> sample frames (floats) to file from the buffer,
       return number actually written. */
int sndSetChans PARG((SOUND *snd, int chans));
int sndGetChans PARG((SOUND *snd));
    /* set number of chans for output, read back for input and output */
int sndSetUchans PARG((SOUND *snd, int chans));
int sndGetUchans PARG((SOUND *snd));
    /* set `user' chans i.e. the number of channels that sndRead returns, 
       regardless of the actual number of channels in the sound */
int sndSetFormat PARG((SOUND *snd, int format));
int sndGetFormat PARG((SOUND *snd));
    /* set on-disk format for output, query on-disk format for in/output */
int sndSetUformat PARG((SOUND *snd, int format));
int sndGetUformat PARG((SOUND *snd));
    /* set `user' format i.e. what sndRead returns or what sndWrite accepts */
long sndSetFrames PARG((SOUND *snd, long frames));
long sndGetFrames PARG((SOUND *snd));
    /* total number of frames in the file */
double sndSetSrate PARG((SOUND *snd, double srate));
double sndGetSrate PARG((SOUND *snd));
    /* sampling rate of the file */
long sndFrameTell PARG((SOUND *snd));
    /* return frame index of next sample frame to be read or written */
long sndFrameSeek PARG((SOUND *snd, long frameoffset, int whence));
    /* seek to a certain point in the file.  Args as fseek(3s) */
int sndFeof PARG((SOUND *snd));
    /* non-zero if we have hit eof on reading a soundfile */
int sndBytesPerFrame PARG((SOUND *snd));
    /* bytes for each sample frame, in the user-visible format */

#endif /* _SND_H */
